@@ -16,6 +16,7 @@
 
 
 import datetime
+import warnings
 
 from oslo_utils import timeutils
 
@@ -37,11 +38,23 @@ class AccessInfo(dict):
     @classmethod
     def factory(cls, resp=None, body=None, region_name=None, auth_token=None,
                 **kwargs):
-        """Create AccessInfo object given a successful auth response & body
-           or a user-provided dict.
+        """Factory function to create a new AccessInfo object.
+
+        Create AccessInfo object given a successful auth response & body
+        or a user-provided dict.
+
+        .. warning::
+
+            Use of the region_name argument is deprecated as of the 1.7.0
+            release and may be removed in the 2.0.0 release.
+
         """
-        # FIXME(jamielennox): Passing region_name is deprecated. Provide an
-        # appropriate warning.
+        if region_name:
+            warnings.warn(
+                'Use of the region_name argument is deprecated as of the '
+                '1.7.0 release and may be removed in the 2.0.0 release.',
+                DeprecationWarning)
+
         auth_ref = None
 
         if body is not None or len(kwargs):
@@ -83,7 +96,7 @@ class AccessInfo(dict):
         return self.get('region_name')
 
     def will_expire_soon(self, stale_duration=None):
-        """Determines if expiration is about to occur.
+        """Determine if expiration is about to occur.
 
         :returns: true if expiration is within the given duration
         :rtype: boolean
@@ -100,8 +113,9 @@ class AccessInfo(dict):
 
     @classmethod
     def is_valid(cls, body, **kwargs):
-        """Determines if processing v2 or v3 token given a successful
-        auth body or a user-provided dict.
+        """Determine if processing valid v2 or v3 token.
+
+        Validates from the auth body or a user-provided dict.
 
         :returns: true if auth body matches implementing class
         :rtype: boolean
@@ -109,7 +123,7 @@ class AccessInfo(dict):
         raise NotImplementedError()
 
     def has_service_catalog(self):
-        """Returns true if the authorization token has a service catalog.
+        """Return true if the authorization token has a service catalog.
 
         :returns: boolean
         """
@@ -117,8 +131,9 @@ class AccessInfo(dict):
 
     @property
     def auth_token(self):
-        """Returns the token_id associated with the auth request, to be used
-        in headers for authenticating OpenStack API requests.
+        """Return the token_id associated with the auth request.
+
+        To be used in headers for authenticating OpenStack API requests.
 
         :returns: str
         """
@@ -132,12 +147,12 @@ class AccessInfo(dict):
     def auth_token(self):
         try:
             del self['auth_token']
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): 'auth_token' is not in the dict
             pass
 
     @property
     def expires(self):
-        """Returns the token expiration (as datetime object)
+        """Return the token expiration (as datetime object).
 
         :returns: datetime
         """
@@ -145,7 +160,7 @@ class AccessInfo(dict):
 
     @property
     def issued(self):
-        """Returns the token issue time (as datetime object)
+        """Return the token issue time (as datetime object).
 
         :returns: datetime
         """
@@ -153,7 +168,8 @@ class AccessInfo(dict):
 
     @property
     def username(self):
-        """Returns the username associated with the authentication request.
+        """Return the username associated with the auth request.
+
         Follows the pattern defined in the V2 API of first looking for 'name',
         returning that if available, and falling back to 'username' if name
         is unavailable.
@@ -164,7 +180,7 @@ class AccessInfo(dict):
 
     @property
     def user_id(self):
-        """Returns the user id associated with the authentication request.
+        """Return the user id associated with the auth request.
 
         :returns: str
         """
@@ -172,8 +188,7 @@ class AccessInfo(dict):
 
     @property
     def user_domain_id(self):
-        """Returns the domain id of the user associated with the authentication
-        request.
+        """Return the user's domain id associated with the auth request.
 
         For v2, it always returns 'default' which may be different from the
         Keystone configuration.
@@ -184,8 +199,7 @@ class AccessInfo(dict):
 
     @property
     def user_domain_name(self):
-        """Returns the domain name of the user associated with the
-        authentication request.
+        """Return the user's  domain name associated with the auth request.
 
         For v2, it always returns 'Default' which may be different from the
         Keystone configuration.
@@ -196,8 +210,7 @@ class AccessInfo(dict):
 
     @property
     def role_ids(self):
-        """Returns a list of role ids of the user associated with the
-        authentication request.
+        """Return a list of user's role ids associated with the auth request.
 
         :returns: a list of strings of role ids
         """
@@ -205,8 +218,7 @@ class AccessInfo(dict):
 
     @property
     def role_names(self):
-        """Returns a list of role names of the user associated with the
-        authentication request.
+        """Return a list of user's role names associated with the auth request.
 
         :returns: a list of strings of role names
         """
@@ -214,7 +226,7 @@ class AccessInfo(dict):
 
     @property
     def domain_name(self):
-        """Returns the domain name associated with the authentication token.
+        """Return the domain name associated with the auth request.
 
         :returns: str or None (if no domain associated with the token)
         """
@@ -222,7 +234,7 @@ class AccessInfo(dict):
 
     @property
     def domain_id(self):
-        """Returns the domain id associated with the authentication token.
+        """Return the domain id associated with the auth request.
 
         :returns: str or None (if no domain associated with the token)
         """
@@ -230,7 +242,7 @@ class AccessInfo(dict):
 
     @property
     def project_name(self):
-        """Returns the project name associated with the authentication request.
+        """Return the project name associated with the auth request.
 
         :returns: str or None (if no project associated with the token)
         """
@@ -243,10 +255,15 @@ class AccessInfo(dict):
 
     @property
     def scoped(self):
-        """Returns true if the authorization token was scoped to a tenant
-           (project), and contains a populated service catalog.
+        """Return true if the auth token was scoped.
 
-           This is deprecated, use project_scoped instead.
+        Return true if scoped to a tenant(project) or domain,
+        and contains a populated service catalog.
+
+        .. warning::
+
+            This is deprecated as of the 1.7.0 release in favor of
+            project_scoped and may be removed in the 2.0.0 release.
 
         :returns: bool
         """
@@ -254,8 +271,7 @@ class AccessInfo(dict):
 
     @property
     def project_scoped(self):
-        """Returns true if the authorization token was scoped to a tenant
-           (project).
+        """Return true if the auth token was scoped to a tenant(project).
 
         :returns: bool
         """
@@ -263,7 +279,7 @@ class AccessInfo(dict):
 
     @property
     def domain_scoped(self):
-        """Returns true if the authorization token was scoped to a domain.
+        """Return true if the auth token was scoped to a domain.
 
         :returns: bool
         """
@@ -271,7 +287,7 @@ class AccessInfo(dict):
 
     @property
     def trust_id(self):
-        """Returns the trust id associated with the authentication token.
+        """Return the trust id associated with the auth request.
 
         :returns: str or None (if no trust associated with the token)
         """
@@ -279,8 +295,9 @@ class AccessInfo(dict):
 
     @property
     def trust_scoped(self):
-        """Returns true if the authorization token was scoped as delegated in a
-        trust, via the OS-TRUST v3 extension.
+        """Return true if the auth token was scoped from a delegated trust.
+
+        The trust delegation is via the OS-TRUST v3 extension.
 
         :returns: bool
         """
@@ -288,7 +305,7 @@ class AccessInfo(dict):
 
     @property
     def trustee_user_id(self):
-        """Returns the trustee user id associated with a trust.
+        """Return the trustee user id associated with a trust.
 
         :returns: str or None (if no trust associated with the token)
         """
@@ -296,7 +313,7 @@ class AccessInfo(dict):
 
     @property
     def trustor_user_id(self):
-        """Returns the trustor user id associated with a trust.
+        """Return the trustor user id associated with a trust.
 
         :returns: str or None (if no trust associated with the token)
         """
@@ -304,9 +321,9 @@ class AccessInfo(dict):
 
     @property
     def project_id(self):
-        """Returns the project ID associated with the authentication
-        request, or None if the authentication request wasn't scoped to a
-        project.
+        """Return the project ID associated with the auth request.
+
+        This returns None if the auth token wasn't scoped to a project.
 
         :returns: str or None (if no project associated with the token)
         """
@@ -319,8 +336,7 @@ class AccessInfo(dict):
 
     @property
     def project_domain_id(self):
-        """Returns the domain id of the project associated with the
-        authentication request.
+        """Return the project's domain id associated with the auth request.
 
         For v2, it returns 'default' if a project is scoped or None which may
         be different from the keystone configuration.
@@ -331,8 +347,7 @@ class AccessInfo(dict):
 
     @property
     def project_domain_name(self):
-        """Returns the domain name of the project associated with the
-        authentication request.
+        """Return the project's domain name associated with the auth request.
 
         For v2, it returns 'Default' if a project is scoped or None  which may
         be different from the keystone configuration.
@@ -343,13 +358,16 @@ class AccessInfo(dict):
 
     @property
     def auth_url(self):
-        """Returns a tuple of URLs from publicURL and adminURL for the service
+        """Return a tuple of identity URLs.
+
+        The identity URLs are from publicURL and adminURL for the service
         'identity' from the service catalog associated with the authorization
         request. If the authentication request wasn't scoped to a tenant
         (project), this property will return None.
 
         DEPRECATED: this doesn't correctly handle region name. You should fetch
-        it from the service catalog yourself.
+        it from the service catalog yourself. This may be removed in the 2.0.0
+        release.
 
         :returns: tuple of urls
         """
@@ -357,12 +375,15 @@ class AccessInfo(dict):
 
     @property
     def management_url(self):
-        """Returns the first adminURL for 'identity' from the service catalog
+        """Return the first adminURL of the identity endpoint.
+
+        The identity endpoint is from the service catalog
         associated with the authorization request, or None if the
         authentication request wasn't scoped to a tenant (project).
 
         DEPRECATED: this doesn't correctly handle region name. You should fetch
-        it from the service catalog yourself.
+        it from the service catalog yourself. This may be removed in the 2.0.0
+        release.
 
         :returns: tuple of urls
         """
@@ -370,7 +391,7 @@ class AccessInfo(dict):
 
     @property
     def version(self):
-        """Returns the version of the auth token from identity service.
+        """Return the version of the auth token from identity service.
 
         :returns: str
         """
@@ -394,17 +415,44 @@ class AccessInfo(dict):
 
     @property
     def is_federated(self):
-        """Returns true if federation was used to get the token.
+        """Return true if federation was used to get the token.
 
         :returns: boolean
         """
         raise NotImplementedError()
 
+    @property
+    def audit_id(self):
+        """Return the audit ID if present.
+
+        :returns: str or None.
+        """
+        raise NotImplementedError()
+
+    @property
+    def audit_chain_id(self):
+        """Return the audit chain ID if present.
+
+        In the event that a token was rescoped then this ID will be the
+        :py:attr:`audit_id` of the initial token. Returns None if no value
+        present.
+
+        :returns: str or None.
+        """
+        raise NotImplementedError()
+
+    @property
+    def initial_audit_id(self):
+        """The audit ID of the initially requested token.
+
+        This is the :py:attr:`audit_chain_id` if present or the
+        :py:attr:`audit_id`.
+        """
+        return self.audit_chain_id or self.audit_id
+
 
 class AccessInfoV2(AccessInfo):
-    """An object for encapsulating a raw v2 auth token from identity
-       service.
-    """
+    """An object for encapsulating raw v2 auth token from identity service."""
 
     def __init__(self, *args, **kwargs):
         super(AccessInfo, self).__init__(*args, **kwargs)
@@ -477,7 +525,8 @@ class AccessInfoV2(AccessInfo):
     def project_name(self):
         try:
             tenant_dict = self['token']['tenant']
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): no 'token' key or 'tenant' key in
+            # token, return the name of the tenant or None
             pass
         else:
             return tenant_dict.get('name')
@@ -485,17 +534,28 @@ class AccessInfoV2(AccessInfo):
         # pre grizzly
         try:
             return self['user']['tenantName']
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): no 'user' key or 'tenantName' in
+            # 'user', attempt 'tenantId' or return None
             pass
 
         # pre diablo, keystone only provided a tenantId
         try:
             return self['token']['tenantId']
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): no 'token' key or 'tenantName' or
+            # 'tenantId' could be found, return None
             pass
 
     @property
     def scoped(self):
+        """Deprecated as of the 1.7.0 release.
+
+        Use project_scoped instead. It may be removed in the
+        2.0.0 release.
+        """
+        warnings.warn(
+            'scoped is deprecated as of the 1.7.0 release in favor of '
+            'project_scoped and may be removed in the 2.0.0 release.',
+            DeprecationWarning)
         if ('serviceCatalog' in self
                 and self['serviceCatalog']
                 and 'tenant' in self['token']):
@@ -531,7 +591,8 @@ class AccessInfoV2(AccessInfo):
     def project_id(self):
         try:
             tenant_dict = self['token']['tenant']
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): no 'token' key or 'tenant' dict,
+            # attempt to return 'tenantId' or return None
             pass
         else:
             return tenant_dict.get('id')
@@ -539,13 +600,15 @@ class AccessInfoV2(AccessInfo):
         # pre grizzly
         try:
             return self['user']['tenantId']
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): no 'user' key or 'tenantId' in
+            # 'user', attempt to retrive from 'token' or return None
             pass
 
         # pre diablo
         try:
             return self['token']['tenantId']
-        except KeyError:
+        except KeyError:  # nosec(cjschaef): no 'token' key or 'tenantId'
+            # could be found, return None
             pass
 
     @property
@@ -560,8 +623,15 @@ class AccessInfoV2(AccessInfo):
 
     @property
     def auth_url(self):
-        # FIXME(jamielennox): this is deprecated in favour of retrieving it
-        # from the service catalog. Provide a warning.
+        """Deprecated as of the 1.7.0 release.
+
+        Use service_catalog.get_urls() instead. It may be removed in the
+        2.0.0 release.
+        """
+        warnings.warn(
+            'auth_url is deprecated as of the 1.7.0 release in favor of '
+            'service_catalog.get_urls() and may be removed in the 2.0.0 '
+            'release.', DeprecationWarning)
         if self.service_catalog:
             return self.service_catalog.get_urls(service_type='identity',
                                                  endpoint_type='publicURL',
@@ -571,8 +641,15 @@ class AccessInfoV2(AccessInfo):
 
     @property
     def management_url(self):
-        # FIXME(jamielennox): this is deprecated in favour of retrieving it
-        # from the service catalog. Provide a warning.
+        """Deprecated as of the 1.7.0 release.
+
+        Use service_catalog.get_urls() instead. It may be removed in the
+        2.0.0 release.
+        """
+        warnings.warn(
+            'management_url is deprecated as of the 1.7.0 release in favor of '
+            'service_catalog.get_urls() and may be removed in the 2.0.0 '
+            'release.', DeprecationWarning)
         if self.service_catalog:
             return self.service_catalog.get_urls(service_type='identity',
                                                  endpoint_type='adminURL',
@@ -592,11 +669,23 @@ class AccessInfoV2(AccessInfo):
     def is_federated(self):
         return False
 
+    @property
+    def audit_id(self):
+        try:
+            return self['token'].get('audit_ids', [])[0]
+        except IndexError:
+            return None
+
+    @property
+    def audit_chain_id(self):
+        try:
+            return self['token'].get('audit_ids', [])[1]
+        except IndexError:
+            return None
+
 
 class AccessInfoV3(AccessInfo):
-    """An object for encapsulating a raw v3 auth token from identity
-       service.
-    """
+    """An object encapsulating raw v3 auth token from identity service."""
 
     def __init__(self, token, *args, **kwargs):
         super(AccessInfo, self).__init__(*args, **kwargs)
@@ -704,6 +793,15 @@ class AccessInfoV3(AccessInfo):
 
     @property
     def scoped(self):
+        """Deprecated as of the 1.7.0 release.
+
+        Use project_scoped instead. It may be removed in the
+        2.0.0 release.
+        """
+        warnings.warn(
+            'scoped is deprecated as of the 1.7.0 release in favor of '
+            'project_scoped and may be removed in the 2.0.0 release.',
+            DeprecationWarning)
         return ('catalog' in self and self['catalog'] and 'project' in self)
 
     @property
@@ -732,8 +830,15 @@ class AccessInfoV3(AccessInfo):
 
     @property
     def auth_url(self):
-        # FIXME(jamielennox): this is deprecated in favour of retrieving it
-        # from the service catalog. Provide a warning.
+        """Deprecated as of the 1.7.0 release.
+
+        Use service_catalog.get_urls() instead. It may be removed in the
+        2.0.0 release.
+        """
+        warnings.warn(
+            'auth_url is deprecated as of the 1.7.0 release in favor of '
+            'service_catalog.get_urls() and may be removed in the 2.0.0 '
+            'release.', DeprecationWarning)
         if self.service_catalog:
             return self.service_catalog.get_urls(service_type='identity',
                                                  endpoint_type='public',
@@ -743,8 +848,15 @@ class AccessInfoV3(AccessInfo):
 
     @property
     def management_url(self):
-        # FIXME(jamielennox): this is deprecated in favour of retrieving it
-        # from the service catalog. Provide a warning.
+        """Deprecated as of the 1.7.0 release.
+
+        Use service_catalog.get_urls() instead. It may be removed in the
+        2.0.0 release.
+        """
+        warnings.warn(
+            'management_url is deprecated as of the 1.7.0 release in favor of '
+            'service_catalog.get_urls() and may be removed in the 2.0.0 '
+            'release.', DeprecationWarning)
         if self.service_catalog:
             return self.service_catalog.get_urls(service_type='identity',
                                                  endpoint_type='admin',
@@ -760,3 +872,17 @@ class AccessInfoV3(AccessInfo):
     @property
     def oauth_consumer_id(self):
         return self.get('OS-OAUTH1', {}).get('consumer_id')
+
+    @property
+    def audit_id(self):
+        try:
+            return self.get('audit_ids', [])[0]
+        except IndexError:
+            return None
+
+    @property
+    def audit_chain_id(self):
+        try:
+            return self.get('audit_ids', [])[1]
+        except IndexError:
+            return None
